@@ -1,50 +1,52 @@
-use std::{
-    fs::{File, OpenOptions},
-    os::{
-        fd::{AsRawFd, AsFd},
-        unix::{io::OwnedFd, fs::OpenOptionsExt}
-    },
-    path::{Path, PathBuf},
-    collections::HashMap,
-    cmp::min,
-    panic::{self, AssertUnwindSafe}
-};
-use cairo::{ImageSurface, Format, Context, Surface, Rectangle, Antialias};
-use rsvg::{Loader, CairoRenderer, SvgHandle};
-use drm::control::ClipRect;
 use anyhow::{anyhow, Result};
+use cairo::{Antialias, Context, Format, ImageSurface, Surface};
+use chrono::{Local, Locale, Timelike};
+use drm::control::ClipRect;
+use freedesktop_icons::lookup;
 use input::{
-    Libinput, LibinputInterface, Device as InputDevice,
     event::{
-        Event, device::DeviceEvent, EventTrait,
+        device::DeviceEvent,
+        keyboard::{KeyState, KeyboardEvent, KeyboardEventTrait},
         touch::{TouchEvent, TouchEventPosition, TouchEventSlot},
-        keyboard::{KeyboardEvent, KeyboardEventTrait, KeyState}
-    }
+        Event, EventTrait,
+    },
+    Device as InputDevice, Libinput, LibinputInterface,
 };
-use libc::{O_ACCMODE, O_RDONLY, O_RDWR, O_WRONLY, c_char};
 use input_linux::{uinput::UInputHandle, EventKind, Key, SynchronizeKind};
-use input_linux_sys::{uinput_setup, input_id, timeval, input_event};
+use input_linux_sys::{input_event, input_id, timeval, uinput_setup};
+use libc::{c_char, O_ACCMODE, O_RDONLY, O_RDWR, O_WRONLY};
+use librsvg_rebind::{prelude::HandleExt, Handle, Rectangle};
 use nix::{
+    errno::Errno,
     sys::{
-        signal::{Signal, SigSet},
-        epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags}
-    }, 
-    errno::Errno
+        epoll::{Epoll, EpollCreateFlags, EpollEvent, EpollFlags},
+        signal::{SigSet, Signal},
+    },
 };
 use privdrop::PrivDrop;
-use freedesktop_icons::lookup;
+use std::{
+    cmp::min,
+    collections::HashMap,
+    fs::{File, OpenOptions},
+    os::{
+        fd::{AsFd, AsRawFd},
+        unix::{fs::OpenOptionsExt, io::OwnedFd},
+    },
+    panic::{self, AssertUnwindSafe},
+    path::{Path, PathBuf},
+};
 
 mod backlight;
-mod display;
-mod pixel_shift;
-mod fonts;
 mod config;
+mod display;
+mod fonts;
+mod pixel_shift;
 
+use crate::config::ConfigManager;
 use backlight::BacklightManager;
+use config::{ButtonConfig, Config};
 use display::DrmBackend;
 use pixel_shift::{PixelShiftManager, PIXEL_SHIFT_WIDTH_PX};
-use config::{ButtonConfig, Config};
-use crate::config::ConfigManager;
 
 const BUTTON_SPACING_PX: i32 = 16;
 const BUTTON_COLOR_INACTIVE: f64 = 0.200;
@@ -54,8 +56,9 @@ const TIMEOUT_MS: i32 = 10 * 1000;
 
 enum ButtonImage {
     Text(String),
-    Svg(SvgHandle),
-    Bitmap(ImageSurface)
+    Svg(Handle),
+    Bitmap(ImageSurface),
+    Time(String, String),
 }
 
 struct Button {
@@ -65,9 +68,10 @@ struct Button {
     action: Key,
 }
 
-fn try_load_svg(path: impl AsRef<Path>) -> Result<ButtonImage> {
-    let handle = Loader::new().read_path(path)?;
-    Ok(ButtonImage::Svg(handle))
+fn try_load_svg(path: &str) -> Result<ButtonImage> {
+    Ok(ButtonImage::Svg(
+        Handle::from_file(path)?.ok_or(anyhow!("failed to load image"))?,
+    ))
 }
 
 fn try_load_png(path: impl AsRef<Path>) -> Result<ButtonImage> {
@@ -78,11 +82,14 @@ fn try_load_png(path: impl AsRef<Path>) -> Result<ButtonImage> {
     }
     let resized = ImageSurface::create(Format::ARgb32, ICON_SIZE, ICON_SIZE).unwrap();
     let c = Context::new(&resized).unwrap();
-    c.scale(ICON_SIZE as f64 / surf.width() as f64, ICON_SIZE as f64 / surf.height() as f64);
+    c.scale(
+        ICON_SIZE as f64 / surf.width() as f64,
+        ICON_SIZE as f64 / surf.height() as f64,
+    );
     c.set_source_surface(surf, 0.0, 0.0).unwrap();
     c.set_antialias(Antialias::Best);
     c.paint().unwrap();
-    return Ok(ButtonImage::Bitmap(resized));
+    Ok(ButtonImage::Bitmap(resized))
 }
 
 fn try_load_image(name: impl AsRef<str>, theme: Option<impl AsRef<str>>) -> Result<ButtonImage> {
@@ -94,10 +101,17 @@ fn try_load_image(name: impl AsRef<str>, theme: Option<impl AsRef<str>>) -> Resu
         // Freedesktop icons
         let theme = theme.as_ref();
         let candidates = vec![
-            lookup(name).with_cache().with_theme(theme).with_size(ICON_SIZE as u16).force_svg().find(),
-            lookup(name).with_cache().with_theme(theme).with_size(ICON_SIZE as u16).find(),
-            lookup(name).with_cache().with_theme(theme).force_svg().find(),
-            lookup(name).with_cache().with_theme(theme).find(),
+            lookup(name)
+                .with_cache()
+                .with_theme(theme)
+                .with_size(ICON_SIZE as u16)
+                .force_svg()
+                .find(),
+            lookup(name)
+                .with_cache()
+                .with_theme(theme)
+                .force_svg()
+                .find(),
         ];
 
         // .flatten() removes `None` and unwraps `Some` values
@@ -118,7 +132,11 @@ fn try_load_image(name: impl AsRef<str>, theme: Option<impl AsRef<str>>) -> Resu
     for location in locations {
         let result = match location.extension().and_then(|s| s.to_str()) {
             Some("png") => try_load_png(&location),
-            Some("svg") => try_load_svg(&location),
+            Some("svg") => try_load_svg(
+                location
+                    .to_str()
+                    .ok_or(anyhow!("image path is not unicode"))?,
+            ),
             _ => Err(anyhow!("invalid file extension")),
         };
 
@@ -140,8 +158,14 @@ impl Button {
             Button::new_text(text, cfg.action)
         } else if let Some(icon) = cfg.icon {
             Button::new_icon(&icon, cfg.theme, cfg.action)
+        } else if let Some(time) = cfg.time {
+            let locale = match cfg.locale {
+                 Some(l) => l,
+                 None => "POSIX".to_string()
+            };
+            Button::new_time(cfg.action, time, locale)
         } else {
-            panic!("Invalid config, a button must have either Text or Icon")
+            panic!("Invalid config, a button must have either Text, Icon or Time")
         }
     }
     fn new_text(text: String, action: Key) -> Button {
@@ -155,40 +179,91 @@ impl Button {
     fn new_icon(path: impl AsRef<str>, theme: Option<impl AsRef<str>>, action: Key) -> Button {
         let image = try_load_image(path, theme).expect("failed to load icon");
         Button {
-            action, image,
+            action,
+            image,
             active: false,
             changed: false,
         }
     }
-    fn render(&self, c: &Context, height: i32, button_left_edge: f64, button_width: u64, y_shift: f64) {
+
+    fn new_time(action: Key, format: String, locale: String) -> Button {
+        Button {
+            action,
+            active: false,
+            changed: false,
+            image: ButtonImage::Time(format, locale),
+        }
+    }
+    fn render(
+        &self,
+        c: &Context,
+        height: i32,
+        button_left_edge: f64,
+        button_width: u64,
+        y_shift: f64,
+    ) {
         match &self.image {
             ButtonImage::Text(text) => {
                 let extents = c.text_extents(text).unwrap();
                 c.move_to(
                     button_left_edge + (button_width as f64 / 2.0 - extents.width() / 2.0).round(),
-                    y_shift + (height as f64 / 2.0 + extents.height() / 2.0).round()
+                    y_shift + (height as f64 / 2.0 + extents.height() / 2.0).round(),
                 );
                 c.show_text(text).unwrap();
-            },
+            }
             ButtonImage::Svg(svg) => {
-                let renderer = CairoRenderer::new(&svg);
-                let x = button_left_edge + (button_width as f64 / 2.0 - (ICON_SIZE / 2) as f64).round();
+                let x =
+                    button_left_edge + (button_width as f64 / 2.0 - (ICON_SIZE / 2) as f64).round();
                 let y = y_shift + ((height as f64 - ICON_SIZE as f64) / 2.0).round();
 
-                renderer.render_document(c,
-                    &Rectangle::new(x, y, ICON_SIZE as f64, ICON_SIZE as f64)
-                ).unwrap();
+                svg.render_document(c, &Rectangle::new(x, y, ICON_SIZE as f64, ICON_SIZE as f64))
+                    .unwrap();
             }
             ButtonImage::Bitmap(surf) => {
-                let x = button_left_edge + (button_width as f64 / 2.0 - (ICON_SIZE / 2) as f64).round();
+                let x =
+                    button_left_edge + (button_width as f64 / 2.0 - (ICON_SIZE / 2) as f64).round();
                 let y = y_shift + ((height as f64 - ICON_SIZE as f64) / 2.0).round();
                 c.set_source_surface(surf, x, y).unwrap();
                 c.rectangle(x, y, ICON_SIZE as f64, ICON_SIZE as f64);
                 c.fill().unwrap();
             }
+            ButtonImage::Time(format, locale) => {
+                 let current_time = Local::now();
+                 let current_locale = Locale::try_from(locale.as_str()).unwrap_or(Locale::POSIX);
+                 let formatted_time;
+                 if format == "24hr" {
+                     formatted_time = format!(
+                     "{}:{}    {} {} {}",
+                      current_time.format_localized("%H", current_locale),
+                      current_time.format_localized("%M", current_locale),
+                      current_time.format_localized("%a", current_locale),
+                      current_time.format_localized("%-e", current_locale),
+                      current_time.format_localized("%b", current_locale)
+                 );
+                 } else {
+                     formatted_time = format!(
+                     "{}:{} {}    {} {} {}",
+                     current_time.format_localized("%-l", current_locale),
+                     current_time.format_localized("%M", current_locale),
+                     current_time.format_localized("%p", current_locale),
+                     current_time.format_localized("%a", current_locale),
+                     current_time.format_localized("%-e", current_locale),
+                     current_time.format_localized("%b", current_locale)
+                 );
+                 }
+                 let time_extents = c.text_extents(&formatted_time).unwrap();
+                 c.move_to(
+                     button_left_edge + (button_width as f64 / 2.0 - time_extents.width() / 2.0).round(),
+                     y_shift + (height as f64 / 2.0 + time_extents.height() / 2.0).round()
+                 );
+                 c.show_text(&formatted_time).unwrap();
+             }
         }
     }
-    fn set_active<F>(&mut self, uinput: &mut UInputHandle<F>, active: bool) where F: AsRawFd {
+    fn set_active<F>(&mut self, uinput: &mut UInputHandle<F>, active: bool)
+    where
+        F: AsRawFd,
+    {
         if self.active != active {
             self.active = active;
             self.changed = true;
@@ -209,24 +284,38 @@ impl FunctionLayer {
         if cfg.is_empty() {
             panic!("Invalid configuration, layer has 0 buttons");
         }
-        
+
         let mut virtual_button_count = 0;
         FunctionLayer {
-            buttons: cfg.into_iter().scan(&mut virtual_button_count, |state, cfg| {
-                let i = **state;
-                let mut stretch = cfg.stretch.unwrap_or(1);
-                if stretch < 1 {
-                    println!("Stretch value must be at least 1, setting to 1.");
-                    stretch = 1;
-                }
-                **state += stretch;
-                Some((i, Button::with_config(cfg)))
-            }).collect(),
+            buttons: cfg
+                .into_iter()
+                .scan(&mut virtual_button_count, |state, cfg| {
+                    let i = **state;
+                    let mut stretch = cfg.stretch.unwrap_or(1);
+                    if stretch < 1 {
+                        println!("Stretch value must be at least 1, setting to 1.");
+                        stretch = 1;
+                    }
+                    if cfg.time.is_some() {
+                        stretch = stretch * 3;
+                    }
+                    **state += stretch;
+                    Some((i, Button::with_config(cfg)))
+                })
+                .collect(),
             virtual_button_count,
         }
     }
-    fn draw(&mut self, config: &Config, width: i32, height: i32, surface: &Surface, pixel_shift: (f64, f64), complete_redraw: bool) -> Vec<ClipRect> {
-        let c = Context::new(&surface).unwrap();
+    fn draw(
+        &mut self,
+        config: &Config,
+        width: i32,
+        height: i32,
+        surface: &Surface,
+        pixel_shift: (f64, f64),
+        complete_redraw: bool,
+    ) -> Vec<ClipRect> {
+        let c = Context::new(surface).unwrap();
         let mut modified_regions = if complete_redraw {
             vec![ClipRect::new(0, 0, height as u16, width as u16)]
         } else {
@@ -234,8 +323,15 @@ impl FunctionLayer {
         };
         c.translate(height as f64, 0.0);
         c.rotate((90.0f64).to_radians());
-        let pixel_shift_width = if config.enable_pixel_shift { PIXEL_SHIFT_WIDTH_PX } else { 0 };
-        let virtual_button_width = ((width - pixel_shift_width as i32) - (BUTTON_SPACING_PX * (self.virtual_button_count - 1) as i32)) as f64 / self.virtual_button_count as f64;
+        let pixel_shift_width = if config.enable_pixel_shift {
+            PIXEL_SHIFT_WIDTH_PX
+        } else {
+            0
+        };
+        let virtual_button_width = ((width - pixel_shift_width as i32)
+            - (BUTTON_SPACING_PX * (self.virtual_button_count - 1) as i32))
+            as f64
+            / self.virtual_button_count as f64;
         let radius = 8.0f64;
         let bot = (height as f64) * 0.15;
         let top = (height as f64) * 0.85;
@@ -256,14 +352,19 @@ impl FunctionLayer {
             };
             let (start, button) = &mut self.buttons[i];
             let start = *start;
-            
+
             if !button.changed && !complete_redraw {
                 continue;
             };
 
-            let left_edge = (start as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64)).floor() + pixel_shift_x + (pixel_shift_width / 2) as f64;
+            let left_edge = (start as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64))
+                .floor()
+                + pixel_shift_x
+                + (pixel_shift_width / 2) as f64;
 
-            let button_width = virtual_button_width + ((end - start - 1) as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64)).floor();
+            let button_width = virtual_button_width
+                + ((end - start - 1) as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64))
+                    .floor();
 
             let color = if button.active {
                 BUTTON_COLOR_ACTIVE
@@ -274,7 +375,12 @@ impl FunctionLayer {
             };
             if !complete_redraw {
                 c.set_source_rgb(0.0, 0.0, 0.0);
-                c.rectangle(left_edge, bot - radius, button_width, top - bot + radius * 2.0);
+                c.rectangle(
+                    left_edge,
+                    bot - radius,
+                    button_width,
+                    top - bot + radius * 2.0,
+                );
                 c.fill().unwrap();
             }
             c.set_source_rgb(color, color, color);
@@ -314,7 +420,13 @@ impl FunctionLayer {
 
             c.fill().unwrap();
             c.set_source_rgb(1.0, 1.0, 1.0);
-            button.render(&c, height, left_edge, button_width.ceil() as u64, pixel_shift_y);
+            button.render(
+                &c,
+                height,
+                left_edge,
+                button_width.ceil() as u64,
+                pixel_shift_y,
+            );
 
             button.changed = false;
 
@@ -323,41 +435,52 @@ impl FunctionLayer {
                     height as u16 - top as u16 - radius as u16,
                     left_edge as u16,
                     height as u16 - bot as u16 + radius as u16,
-                    left_edge as u16 + button_width as u16
+                    left_edge as u16 + button_width as u16,
                 ));
             }
         }
 
         modified_regions
     }
-    
+
     fn hit(&self, width: u16, height: u16, x: f64, y: f64, i: Option<usize>) -> Option<usize> {
-        let virtual_button_width = (width as i32 - (BUTTON_SPACING_PX * (self.virtual_button_count - 1) as i32)) as f64 / self.virtual_button_count as f64;
-        
+        let virtual_button_width =
+            (width as i32 - (BUTTON_SPACING_PX * (self.virtual_button_count - 1) as i32)) as f64
+                / self.virtual_button_count as f64;
+
         let i = i.unwrap_or_else(|| {
             let virtual_i = (x / (width as f64 / self.virtual_button_count as f64)) as usize;
-            self.buttons.iter().position(|(start, _)| *start > virtual_i).unwrap_or(self.buttons.len()) - 1
+            self.buttons
+                .iter()
+                .position(|(start, _)| *start > virtual_i)
+                .unwrap_or(self.buttons.len())
+                - 1
         });
         if i >= self.buttons.len() {
             return None;
         }
-        
+
         let start = self.buttons[i].0;
         let end = if i + 1 < self.buttons.len() {
             self.buttons[i + 1].0
         } else {
             self.virtual_button_count
         };
-        
+
         let left_edge = (start as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64)).floor();
 
-        let button_width = virtual_button_width + ((end - start - 1) as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64)).floor();
-        
-        if x < left_edge || x > (left_edge + button_width)
-            || y < 0.1 * height as f64 || y > 0.9 * height as f64 {
+        let button_width = virtual_button_width
+            + ((end - start - 1) as f64 * (virtual_button_width + BUTTON_SPACING_PX as f64))
+                .floor();
+
+        if x < left_edge
+            || x > (left_edge + button_width)
+            || y < 0.1 * height as f64
+            || y > 0.9 * height as f64
+        {
             return None;
         }
-        
+
         Some(i)
     }
 }
@@ -381,30 +504,40 @@ impl LibinputInterface for Interface {
     }
 }
 
-
-fn emit<F>(uinput: &mut UInputHandle<F>, ty: EventKind, code: u16, value: i32) where F: AsRawFd {
-    uinput.write(&[input_event {
-        value: value,
-        type_: ty as u16,
-        code: code,
-        time: timeval {
-            tv_sec: 0,
-            tv_usec: 0
-        }
-    }]).unwrap();
+fn emit<F>(uinput: &mut UInputHandle<F>, ty: EventKind, code: u16, value: i32)
+where
+    F: AsRawFd,
+{
+    uinput
+        .write(&[input_event {
+            value,
+            type_: ty as u16,
+            code,
+            time: timeval {
+                tv_sec: 0,
+                tv_usec: 0,
+            },
+        }])
+        .unwrap();
 }
 
-fn toggle_key<F>(uinput: &mut UInputHandle<F>, code: Key, value: i32) where F: AsRawFd {
+fn toggle_key<F>(uinput: &mut UInputHandle<F>, code: Key, value: i32)
+where
+    F: AsRawFd,
+{
     emit(uinput, EventKind::Key, code as u16, value);
-    emit(uinput, EventKind::Synchronize, SynchronizeKind::Report as u16, 0);
+    emit(
+        uinput,
+        EventKind::Synchronize,
+        SynchronizeKind::Report as u16,
+        0,
+    );
 }
 
 fn main() {
     let mut drm = DrmBackend::open_card().unwrap();
     let (height, width) = drm.mode().size();
-    let _ = panic::catch_unwind(AssertUnwindSafe(|| {
-        real_main(&mut drm)
-    }));
+    let _ = panic::catch_unwind(AssertUnwindSafe(|| real_main(&mut drm)));
     let crash_bitmap = include_bytes!("crash_bitmap.raw");
     let mut map = drm.map().unwrap();
     let data = map.as_mut();
@@ -421,7 +554,7 @@ fn main() {
         }
     }
     drop(map);
-    drm.dirty(&[ClipRect::new(0, 0, height as u16, width as u16)]).unwrap();
+    drm.dirty(&[ClipRect::new(0, 0, height, width)]).unwrap();
     let mut sigset = SigSet::empty();
     sigset.add(Signal::SIGTERM);
     sigset.wait().unwrap();
@@ -432,6 +565,7 @@ fn real_main(drm: &mut DrmBackend) {
     let (db_width, db_height) = drm.fb_info().unwrap().size();
     let mut uinput = UInputHandle::new(OpenOptions::new().write(true).open("/dev/uinput").unwrap());
     let mut backlight = BacklightManager::new();
+    let mut last_redraw_minute = Local::now().minute();
     let mut cfg_mgr = ConfigManager::new();
     let (mut cfg, mut layers) = cfg_mgr.load_config(width);
     let mut pixel_shift = PixelShiftManager::new();
@@ -443,9 +577,10 @@ fn real_main(drm: &mut DrmBackend) {
         .user("nobody")
         .group_list(&groups)
         .apply()
-        .unwrap_or_else(|e| { panic!("Failed to drop privileges: {}", e) });
+        .unwrap_or_else(|e| panic!("Failed to drop privileges: {}", e));
 
-    let mut surface = ImageSurface::create(Format::ARgb32, db_width as i32, db_height as i32).unwrap();
+    let mut surface =
+        ImageSurface::create(Format::ARgb32, db_width as i32, db_height as i32).unwrap();
     let mut active_layer = 0;
     let mut needs_complete_redraw = true;
 
@@ -454,9 +589,15 @@ fn real_main(drm: &mut DrmBackend) {
     input_tb.udev_assign_seat("seat-touchbar").unwrap();
     input_main.udev_assign_seat("seat0").unwrap();
     let epoll = Epoll::new(EpollCreateFlags::empty()).unwrap();
-    epoll.add(input_main.as_fd(), EpollEvent::new(EpollFlags::EPOLLIN, 0)).unwrap();
-    epoll.add(input_tb.as_fd(), EpollEvent::new(EpollFlags::EPOLLIN, 1)).unwrap();
-    epoll.add(cfg_mgr.fd(), EpollEvent::new(EpollFlags::EPOLLIN, 2)).unwrap();
+    epoll
+        .add(input_main.as_fd(), EpollEvent::new(EpollFlags::EPOLLIN, 0))
+        .unwrap();
+    epoll
+        .add(input_tb.as_fd(), EpollEvent::new(EpollFlags::EPOLLIN, 1))
+        .unwrap();
+    epoll
+        .add(cfg_mgr.fd(), EpollEvent::new(EpollFlags::EPOLLIN, 2))
+        .unwrap();
     uinput.set_evbit(EventKind::Key).unwrap();
     for layer in &layers {
         for button in &layer.buttons {
@@ -468,16 +609,18 @@ fn real_main(drm: &mut DrmBackend) {
     for i in 0..dev_name.len() {
         dev_name_c[i] = dev_name[i] as c_char;
     }
-    uinput.dev_setup(&uinput_setup {
-        id: input_id {
-            bustype: 0x19,
-            vendor: 0x1209,
-            product: 0x316E,
-            version: 1
-        },
-        ff_effects_max: 0,
-        name: dev_name_c
-    }).unwrap();
+    uinput
+        .dev_setup(&uinput_setup {
+            id: input_id {
+                bustype: 0x19,
+                vendor: 0x1209,
+                product: 0x316E,
+                version: 1,
+            },
+            ff_effects_max: 0,
+            name: dev_name_c,
+        })
+        .unwrap();
     uinput.dev_create().unwrap();
 
     let mut digitizer: Option<InputDevice> = None;
@@ -488,7 +631,10 @@ fn real_main(drm: &mut DrmBackend) {
             needs_complete_redraw = true;
         }
 
-        let mut next_timeout_ms = TIMEOUT_MS;
+        let now = Local::now();
+        let ms_left = ((60 - now.second()) * 1000) as i32;
+        let mut next_timeout_ms = min(ms_left, TIMEOUT_MS);
+
         if cfg.enable_pixel_shift {
             let (pixel_shift_needs_redraw, pixel_shift_next_timeout_ms) = pixel_shift.update();
             if pixel_shift_needs_redraw {
@@ -497,21 +643,39 @@ fn real_main(drm: &mut DrmBackend) {
             next_timeout_ms = min(next_timeout_ms, pixel_shift_next_timeout_ms);
         }
 
+        let current_minute = now.minute();
+        for button in &mut layers[active_layer].buttons {
+            if matches!(button.1.image, ButtonImage::Time(_, _)) && (current_minute != last_redraw_minute) {
+                 needs_complete_redraw = true;
+                 last_redraw_minute = current_minute;
+            }
+        }
+
         if needs_complete_redraw || layers[active_layer].buttons.iter().any(|b| b.1.changed) {
             let shift = if cfg.enable_pixel_shift {
                 pixel_shift.get()
             } else {
                 (0.0, 0.0)
             };
-            let clips = layers[active_layer].draw(&cfg, width as i32, height as i32, &surface, shift, needs_complete_redraw);
+            let clips = layers[active_layer].draw(
+                &cfg,
+                width as i32,
+                height as i32,
+                &surface,
+                shift,
+                needs_complete_redraw,
+            );
             let data = surface.data().unwrap();
             drm.map().unwrap().as_mut()[..data.len()].copy_from_slice(&data);
             drm.dirty(&clips).unwrap();
             needs_complete_redraw = false;
         }
 
-        match epoll.wait(&mut [EpollEvent::new(EpollFlags::EPOLLIN, 0)], next_timeout_ms as u16) {
-            Err(Errno::EINTR) | Ok(_) => { 0 },
+        match epoll.wait(
+            &mut [EpollEvent::new(EpollFlags::EPOLLIN, 0)],
+            next_timeout_ms as u16,
+        ) {
+            Err(Errno::EINTR) | Ok(_) => 0,
             e => e.unwrap(),
         };
         input_tb.dispatch().unwrap();
@@ -524,22 +688,22 @@ fn real_main(drm: &mut DrmBackend) {
                     if dev.name().contains(" Touch Bar") {
                         digitizer = Some(dev);
                     }
-                },
+                }
                 Event::Keyboard(KeyboardEvent::Key(key)) => {
                     if key.key() == Key::Fn as u32 {
                         let new_layer = match key.key_state() {
                             KeyState::Pressed => 1,
-                            KeyState::Released => 0
+                            KeyState::Released => 0,
                         };
                         if active_layer != new_layer {
                             active_layer = new_layer;
                             needs_complete_redraw = true;
                         }
                     }
-                },
+                }
                 Event::Touch(te) => {
                     if Some(te.device()) != digitizer || backlight.current_bl() == 0 {
-                        continue
+                        continue;
                     }
                     match te {
                         TouchEvent::Down(dn) => {
@@ -547,9 +711,11 @@ fn real_main(drm: &mut DrmBackend) {
                             let y = dn.y_transformed(height as u32);
                             if let Some(btn) = layers[active_layer].hit(width, height, x, y, None) {
                                 touches.insert(dn.seat_slot(), (active_layer, btn));
-                                layers[active_layer].buttons[btn].1.set_active(&mut uinput, true);
+                                layers[active_layer].buttons[btn]
+                                    .1
+                                    .set_active(&mut uinput, true);
                             }
-                        },
+                        }
                         TouchEvent::Motion(mtn) => {
                             if !touches.contains_key(&mtn.seat_slot()) {
                                 continue;
@@ -558,9 +724,11 @@ fn real_main(drm: &mut DrmBackend) {
                             let x = mtn.x_transformed(width as u32);
                             let y = mtn.y_transformed(height as u32);
                             let (layer, btn) = *touches.get(&mtn.seat_slot()).unwrap();
-                            let hit = layers[active_layer].hit(width, height, x, y, Some(btn)).is_some();
+                            let hit = layers[active_layer]
+                                .hit(width, height, x, y, Some(btn))
+                                .is_some();
                             layers[layer].buttons[btn].1.set_active(&mut uinput, hit);
-                        },
+                        }
                         TouchEvent::Up(up) => {
                             if !touches.contains_key(&up.seat_slot()) {
                                 continue;
@@ -570,7 +738,7 @@ fn real_main(drm: &mut DrmBackend) {
                         }
                         _ => {}
                     }
-                },
+                }
                 _ => {}
             }
         }
